/**
 * Copyright (c) 2014,Egret-Labs.org
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Egret-Labs.org nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGRET-LABS.ORG AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL EGRET-LABS.ORG AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var egret;
(function (egret) {
    var gui;
    (function (gui) {
        /**
            * 适用Viewport的滑动类
            */
        var ViewportScroller = (function (_super) {
            __extends(ViewportScroller, _super);
            /**
                * @method egret.gui.GroupBase#constructor
                */
            function ViewportScroller(content) {
                _super.call(this, content);
                this._width = 0;
                this._height = 0;
            }
            ViewportScroller.prototype.setContent = function (content) {
                _super.prototype.setContent.call(this, content);
                var viewport = this._content;
                this._scrollLeft = viewport.horizontalScrollPosition;
                this._scrollTop = viewport.verticalScrollPosition;
            };
            ViewportScroller.prototype._updateContentPosition = function () {
                var content = this._content;
                content.horizontalScrollPosition = this._scrollLeft;
                content.verticalScrollPosition = this._scrollTop;
                content.setLayoutBoundsSize(this._width, this._height);
                this.dispatchEvent(new egret.Event(egret.Event.CHANGE));
            };
            ViewportScroller.prototype.getMaxScrollLeft = function () {
                var content = this._content;
                var max = content.contentWidth - content.width;
                return Math.max(max, 0);
            };
            ViewportScroller.prototype.getMaxScrollTop = function () {
                var content = this._content;
                var max = content.contentHeight - content.height;
                return Math.max(max, 0);
            };
            ViewportScroller.prototype._getContentWidth = function () {
                return this._content.contentWidth;
            };
            ViewportScroller.prototype._getContentHeight = function () {
                return this._content.contentHeight;
            };
            ViewportScroller.prototype._setHeight = function (value) {
                this._explicitHeight = value;
                this._hasHeightSet = egret.NumberUtils.isNumber(value);
                this._height = value;
                var content = this._content;
                content && content.setLayoutBoundsSize(this._width, this._height);
            };
            ViewportScroller.prototype._setWidth = function (value) {
                this._explicitWidth = value;
                this._hasWidthSet = egret.NumberUtils.isNumber(value);
                this._width = value;
                var content = this._content;
                content && content.setLayoutBoundsSize(this._width, this._height);
            };
            Object.defineProperty(ViewportScroller.prototype, "height", {
                get: function () {
                    return this._height;
                },
                set: function (value) {
                    this._setHeight(value);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(ViewportScroller.prototype, "width", {
                get: function () {
                    return this._width;
                },
                set: function (value) {
                    this._setWidth(value);
                },
                enumerable: true,
                configurable: true
            });
            ViewportScroller.prototype.invalidateSize = function () {
                var p = this.parent;
                p && p.invalidateSize();
            };
            ViewportScroller.prototype.invalidateDisplayList = function () {
                var p = this.parent;
                p && p.invalidateDisplayList();
            };
            return ViewportScroller;
        })(egret.ScrollView);
        ViewportScroller.prototype.__class__ = "egret.gui.ViewportScroller";
        /**
         * @class egret.gui.Scroller
         * @classdesc
         * 滚动条组件
         * @extends egret.gui.UIComponent
         * @implements egret.gui.IVisualElementContainer
         */
        var Scroller = (function (_super) {
            __extends(Scroller, _super);
            /**
             * 构造函数
             * @method egret.gui.Scroller#constructor
             */
            function Scroller() {
                _super.call(this);
                this._verticalScrollPolicy = "auto";
                this._horizontalScrollPolicy = "auto";
            }
            /**
             * @method egret.gui.Scroller#measure
             */
            Scroller.prototype.measure = function () {
                if (!this._viewport)
                    return;
                this.measuredWidth = this._viewport.preferredWidth;
                this.measuredHeight = this._viewport.preferredHeight;
            };
            /**
             * @param unscaledWidth {number}
             * @param unscaledHeight {number}
             */
            Scroller.prototype.updateDisplayList = function (unscaledWidth, unscaledHeight) {
                this._scroller._setWidth(unscaledWidth);
                this._scroller._setHeight(unscaledHeight);
                if (this.horizontalScrollBar) {
                    if (this._horizontalScrollPolicy != "off") {
                        this.horizontalScrollBar._setViewportMetric(unscaledWidth, this._viewport.contentWidth);
                        this.horizontalScrollBar._setWidth(unscaledWidth - 2);
                        this.horizontalScrollBar.x = 1;
                        this.horizontalScrollBar.y = unscaledHeight - this.horizontalScrollBar._height - 1;
                    }
                }
                if (this.verticalScrollBar) {
                    if (this._verticalScrollPolicy != "off") {
                        this.verticalScrollBar._setViewportMetric(unscaledHeight, this._viewport.contentHeight);
                        this.verticalScrollBar._setHeight(unscaledHeight - 2);
                        this.verticalScrollBar.y = 1;
                        this.verticalScrollBar.x = unscaledWidth - this.verticalScrollBar.width - 1;
                    }
                }
            };
            Object.defineProperty(Scroller.prototype, "verticalScrollPolicy", {
                /**
                 * 垂直滚动条显示策略，参见ScrollPolicy类定义的常量。
                 * @member egret.gui.Scroller#verticalScrollPolicy
                 */
                get: function () {
                    return this._verticalScrollPolicy;
                },
                set: function (value) {
                    if (value == this._verticalScrollPolicy)
                        return;
                    this._verticalScrollPolicy = value;
                    this._checkVbar();
                    if (this._scroller)
                        this._scroller.verticalScrollPolicy = value;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Scroller.prototype, "horizontalScrollPolicy", {
                /**
                 * 水平滚动条显示策略，参见ScrollPolicy类定义的常量。
                 * @member egret.gui.Scroller#horizontalScrollPolicy
                 */
                get: function () {
                    return this._horizontalScrollPolicy;
                },
                set: function (value) {
                    if (value == this._horizontalScrollPolicy)
                        return;
                    this._horizontalScrollPolicy = value;
                    this._checkHbar();
                    if (this._scroller)
                        this._scroller.horizontalScrollPolicy = value;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Scroller.prototype, "viewport", {
                /**
                 * 要滚动的视域组件。
                 * @member egret.gui.Scroller#viewport
                 */
                get: function () {
                    return this._viewport;
                },
                set: function (value) {
                    if (value == this._viewport)
                        return;
                    this.uninstallViewport();
                    this._viewport = value;
                    this.installViewport();
                    this.dispatchEventWith("viewportChanged");
                },
                enumerable: true,
                configurable: true
            });
            /**
             * 安装并初始化视域组件
             */
            Scroller.prototype.installViewport = function () {
                var viewport = this.viewport;
                if (this._scroller == null) {
                    this._scroller = new ViewportScroller(viewport);
                    this._scroller.horizontalScrollPolicy = this._horizontalScrollPolicy;
                    this._scroller.verticalScrollPolicy = this._verticalScrollPolicy;
                    this._scroller.addEventListener(egret.Event.CHANGE, this._scrollerChangedHandler, this);
                    this._addToDisplayListAt(this._scroller, 0);
                }
                if (viewport) {
                    viewport.clipAndEnableScrolling = true;
                    this._scroller.setContent(viewport);
                    viewport.addEventListener(egret.gui.PropertyChangeEvent.PROPERTY_CHANGE, this._viewportChangedHandler, this);
                    if ("_addToStyleProtoChain" in viewport) {
                        viewport["_addToStyleProtoChain"](this._styleProtoChain);
                    }
                }
            };
            Scroller.prototype._onAddToStage = function () {
                _super.prototype._onAddToStage.call(this);
                this._scroller._stage = this.stage;
            };
            /**
             * 卸载视域组件
             */
            Scroller.prototype.uninstallViewport = function () {
                if (this.viewport) {
                    this.viewport.clipAndEnableScrolling = false;
                    this.viewport.removeEventListener(egret.gui.PropertyChangeEvent.PROPERTY_CHANGE, this._viewportChangedHandler, this);
                }
                if (this._scroller) {
                    this._scroller.removeContent();
                }
            };
            Scroller.prototype._viewportChangedHandler = function (e) {
                if (e.property == "horizontalScrollPosition")
                    this.setViewportHScrollPosition(this.viewport.horizontalScrollPosition);
                if (e.property == "verticalScrollPosition")
                    this.setViewportVScrollPosition(this.viewport.verticalScrollPosition);
            };
            Scroller.prototype._scrollerChangedHandler = function (e) {
                this.setViewportHScrollPosition(this._scroller.scrollLeft);
                this.setViewportVScrollPosition(this._scroller.scrollTop);
            };
            Scroller.prototype.setViewportVScrollPosition = function (pos) {
                if (this._scroller.scrollTop != pos)
                    this._scroller.scrollTop = pos;
                if (this.verticalScrollBar && this.verticalScrollBar.value != pos)
                    this.verticalScrollBar.setPosition(pos);
            };
            Scroller.prototype.setViewportHScrollPosition = function (pos) {
                if (this._scroller.scrollLeft != pos)
                    this._scroller.scrollLeft = pos;
                if (this.horizontalScrollBar && this.horizontalScrollBar.value != pos)
                    this.horizontalScrollBar._setValue(pos);
            };
            /**
             * 缓动到水平滚动位置
             * @method egret.gui.Scroller#throwHorizontally
             * @param hspTo {number}
             * @param duration {number}
             */
            Scroller.prototype.throwHorizontally = function (hspTo, duration) {
                if (duration === void 0) { duration = 500; }
                if (!this._scroller)
                    return;
                this._scroller.setScrollLeft(hspTo, duration);
            };
            /**
             * 缓动到垂直滚动位置
             * @method egret.gui.Scroller#throwVertically
             * @param vspTo {number}
             * @param duration {number}
             */
            Scroller.prototype.throwVertically = function (vspTo, duration) {
                if (duration === void 0) { duration = 500; }
                if (!this._scroller)
                    return;
                this._scroller.setScrollTop(vspTo, duration);
            };
            Object.defineProperty(Scroller.prototype, "numElements", {
                /**
                 * @member egret.gui.Scroller#numElements
                 */
                get: function () {
                    return this.viewport ? 1 : 0;
                },
                enumerable: true,
                configurable: true
            });
            /**
             * 抛出索引越界异常
             */
            Scroller.prototype.throwRangeError = function (index) {
                throw new RangeError("索引:\"" + index + "\"超出可视元素索引范围");
            };
            /**
             * @param index {number}
             * @returns {IVisualElement}
             */
            Scroller.prototype.getElementAt = function (index) {
                if (this.viewport && index == 0)
                    return this.viewport;
                else
                    this.throwRangeError(index);
                return null;
            };
            /**
             * @param element {IVisualElement}
             * @returns {number}
             */
            Scroller.prototype.getElementIndex = function (element) {
                if (element != null && element == this.viewport)
                    return 0;
                else
                    return -1;
            };
            /**
             * @param element {IVisualElement}
             * @returns {boolean}
             */
            Scroller.prototype.containsElement = function (element) {
                if (element != null && element == this.viewport)
                    return true;
                return false;
            };
            Scroller.prototype.throwNotSupportedError = function () {
                throw new Error("此方法在Scroller组件内不可用!");
            };
            /**
             * @deprecated
             * @param element {IVisualElement}
             * @returns {IVisualElement}
             */
            Scroller.prototype.addElement = function (element) {
                this.throwNotSupportedError();
                return null;
            };
            /**
             * @deprecated
             * @param element {IVisualElement}
             * @param index {number}
             * @returns {IVisualElement}
             */
            Scroller.prototype.addElementAt = function (element, index) {
                this.throwNotSupportedError();
                return null;
            };
            /**
             * @deprecated
             * @param element {IVisualElement}
             * @returns {IVisualElement}
             */
            Scroller.prototype.removeElement = function (element) {
                this.throwNotSupportedError();
                return null;
            };
            /**
             * @deprecated
             * @param index {number}
             * @returns {IVisualElement}
             */
            Scroller.prototype.removeElementAt = function (index) {
                this.throwNotSupportedError();
                return null;
            };
            /**
             * @deprecated
             */
            Scroller.prototype.removeAllElements = function () {
                this.throwNotSupportedError();
            };
            /**
             * @deprecated
             * @param element {IVisualElement}
             * @param index {number}
             */
            Scroller.prototype.setElementIndex = function (element, index) {
                this.throwNotSupportedError();
            };
            /**
             * @deprecated
             * @param element1 {IVisualElement}
             * @param element2 {IVisualElement}
             */
            Scroller.prototype.swapElements = function (element1, element2) {
                this.throwNotSupportedError();
            };
            /**
             * @deprecated
             * @param index1 {number}
             * @param index2 {number}
             */
            Scroller.prototype.swapElementsAt = function (index1, index2) {
                this.throwNotSupportedError();
            };
            /**
             * @deprecated
             * @param child {DisplayObject}
             * @returns {DisplayObject}
             */
            Scroller.prototype.addChild = function (child) {
                this.throwNotSupportedError();
                return null;
            };
            /**
             * @deprecated
             * @param child {DisplayObject}
             * @param index {number}
             * @returns {DisplayObject}
             */
            Scroller.prototype.addChildAt = function (child, index) {
                this.throwNotSupportedError();
                return null;
            };
            /**
             * @deprecated
             * @param child {DisplayObject}
             * @returns {DisplayObject}
             */
            Scroller.prototype.removeChild = function (child) {
                this.throwNotSupportedError();
                return null;
            };
            /**
             * @deprecated
             * @param index {number}
             * @returns {DisplayObject}
             */
            Scroller.prototype.removeChildAt = function (index) {
                this.throwNotSupportedError();
                return null;
            };
            /**
             * @deprecated
             * @param child {DisplayObject}
             * @param index {number}
             */
            Scroller.prototype.setChildIndex = function (child, index) {
                this.throwNotSupportedError();
            };
            /**
             * @deprecated
             * @param child1 {DisplayObject}
             * @param child2 {DisplayObject}
             */
            Scroller.prototype.swapChildren = function (child1, child2) {
                this.throwNotSupportedError();
            };
            /**
             * @deprecated
             * @param index1 {number}
             * @param index2 {number}
             */
            Scroller.prototype.swapChildrenAt = function (index1, index2) {
                this.throwNotSupportedError();
            };
            Scroller.prototype._checkHbar = function () {
                if (this._horizontalScrollPolicy == "off") {
                    if (this.horizontalScrollBar) {
                        this._removeFromDisplayList(this.horizontalScrollBar);
                    }
                    return;
                }
                if (!this.horizontalScrollBar)
                    return;
                var bar = this.horizontalScrollBar;
                bar.addEventListener(egret.Event.CHANGE, this.hBarChanged, this, false);
                bar._setViewportMetric(this._viewport.width, this._viewport.contentWidth);
                this.horizontalScrollBar = bar;
                var host = bar.owner;
                if (host && "removeElement" in host) {
                    host.removeElement(bar);
                }
                this._addToDisplayList(this.horizontalScrollBar);
            };
            Scroller.prototype._checkVbar = function () {
                if (this._verticalScrollPolicy == "off") {
                    if (this.verticalScrollBar) {
                        this._removeFromDisplayList(this.verticalScrollBar);
                    }
                    return;
                }
                if (!this.verticalScrollBar)
                    return;
                var vbar = this.verticalScrollBar;
                vbar.addEventListener(egret.Event.CHANGE, this.vBarChanged, this, false);
                vbar._setViewportMetric(this._viewport.height, this._viewport.contentHeight);
                this.verticalScrollBar = vbar;
                var host = vbar.owner;
                if (host && "removeElement" in host) {
                    host.removeElement(vbar);
                }
                this._addToDisplayList(this.verticalScrollBar);
            };
            /**
             * 若皮肤是ISkin,则调用此方法附加皮肤中的公共部件
             * @param partName {string}
             * @param instance {any}
             */
            Scroller.prototype.partAdded = function (partName, instance) {
                _super.prototype.partAdded.call(this, partName, instance);
                if (instance == this.horizontalScrollBar) {
                    this._checkHbar();
                }
                if (instance == this.verticalScrollBar) {
                    this._checkVbar();
                }
            };
            Scroller.prototype._removeScrollBars = function () {
                if (this.horizontalScrollBar) {
                    this._removeFromDisplayList(this.horizontalScrollBar);
                    this.horizontalScrollBar.removeEventListener(egret.Event.CHANGE, this.hBarChanged, this, false);
                    this.horizontalScrollBar = null;
                }
                if (this.verticalScrollBar) {
                    this._removeFromDisplayList(this.verticalScrollBar);
                    this.verticalScrollBar.removeEventListener(egret.Event.CHANGE, this.vBarChanged, this, false);
                    this.verticalScrollBar = null;
                }
            };
            Scroller.prototype.hBarChanged = function (e) {
                this.setViewportHScrollPosition(this.horizontalScrollBar._getValue());
            };
            Scroller.prototype.vBarChanged = function (e) {
                this.setViewportVScrollPosition(this.verticalScrollBar.getPosition());
            };
            Scroller.prototype._createOwnStyleProtoChain = function (chain) {
                chain = _super.prototype._createOwnStyleProtoChain.call(this, chain);
                var viewport = this._viewport;
                if (viewport && "_addToStyleProtoChain" in viewport) {
                    viewport["_addToStyleProtoChain"](chain);
                }
                return chain;
            };
            /**
             * 添加到父级容器的样式原型链
             */
            Scroller.prototype.regenerateStyleCache = function (parentChain) {
                _super.prototype.regenerateStyleCache.call(this, parentChain);
                var viewport = this._viewport;
                if (viewport && "regenerateStyleCache" in viewport) {
                    viewport["regenerateStyleCache"](parentChain);
                }
            };
            /**
             * 通知项列表样式发生改变
             */
            Scroller.prototype.notifyStyleChangeInChildren = function (styleProp) {
                _super.prototype.notifyStyleChangeInChildren.call(this, styleProp);
                var viewport = this._viewport;
                if (viewport && "styleChanged" in viewport) {
                    viewport["styleChanged"](styleProp);
                    viewport["notifyStyleChangeInChildren"](styleProp);
                }
            };
            /**
             * 更新子项的nestLevel属性
             */
            Scroller.prototype._updateChildrenNestLevel = function () {
                _super.prototype._updateChildrenNestLevel.call(this);
                var viewport = this._viewport;
                if (viewport && "nestLevel" in viewport) {
                    viewport["nestLevel"] = this.nestLevel + 1;
                }
            };
            return Scroller;
        })(gui.SkinnableComponent);
        gui.Scroller = Scroller;
        Scroller.prototype.__class__ = "egret.gui.Scroller";
    })(gui = egret.gui || (egret.gui = {}));
})(egret || (egret = {}));
